@@ -1,6 +1,6 @@
 /*
     PowerDNS Versatile Database Driven Nameserver
-    Copyright (C) 2002 - 2005  PowerDNS.COM BV
+    Copyright (C) 2002 - 2008  PowerDNS.COM BV
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2 as 
@@ -27,6 +27,7 @@
 #include <sys/mman.h>
 #include <fcntl.h>
 #include <sys/types.h>
+#include <boost/shared_ptr.hpp>
 
 #include <sys/stat.h>
 #include "ahuexception.hh"
@@ -37,6 +38,7 @@
 #include "statbag.hh"
 #include "misc.hh"
 using namespace std;
+using namespace boost;
 
 ArgvMap &arg()
 {
@@ -52,35 +54,39 @@ int main(int argc, char **argv)
   string localdir;
 
   static char pietje[128]="!@@SYSCONFDIR@@:";
-  arg().set("config-dir","Location of configuration directory (pdns.conf)")=
+  ::arg().set("config-dir","Location of configuration directory (pdns.conf)")=
     strcmp(pietje+1,"@@SYSCONFDIR@@:") ? pietje+strlen("@@SYSCONFDIR@@:")+1 : SYSCONFDIR;
   
-  arg().set("socket-dir","Where the controlsocket will live")=LOCALSTATEDIR;
-  arg().set("config-name","Name of this virtual configuration - will rename the binary image")="";
-  arg().set("chroot","")="";
-  arg().setCmd("help","Provide a helpful message");
-  arg().laxParse(argc,argv);
+  ::arg().set("socket-dir","Where the controlsocket will live")=LOCALSTATEDIR;
+  ::arg().set("remote-address","Remote address to query");
+  ::arg().set("remote-port","Remote port to query")="53000";
+  ::arg().set("secret","Secret needed to connect to remote PowerDNS");
 
-  if(arg().mustDo("help")) {
+  ::arg().set("config-name","Name of this virtual configuration - will rename the binary image")="";
+  ::arg().set("chroot","")="";
+  ::arg().setCmd("help","Provide a helpful message");
+  ::arg().laxParse(argc,argv);
+
+  if(::arg().mustDo("help")) {
     cerr<<"syntax:"<<endl<<endl;
-    cerr<<arg().helpstring(arg()["help"])<<endl;
+    cerr<<::arg().helpstring(::arg()["help"])<<endl;
     exit(99);
   }
 
-  if(arg()["config-name"]!="") 
-    s_programname+="-"+arg()["config-name"];
+  if(::arg()["config-name"]!="") 
+    s_programname+="-"+::arg()["config-name"];
 
-  string configname=arg()["config-dir"]+"/"+s_programname+".conf";
+  string configname=::arg()["config-dir"]+"/"+s_programname+".conf";
   cleanSlashes(configname);
   
-  arg().laxFile(configname.c_str());
-  string socketname=arg()["socket-dir"]+"/"+s_programname+".controlsocket";
-  if(arg()["chroot"].empty())
+  ::arg().laxFile(configname.c_str());
+  string socketname=::arg()["socket-dir"]+"/"+s_programname+".controlsocket";
+  if(::arg()["chroot"].empty())
     localdir="/tmp";
   else
     localdir=dirname(strdup(socketname.c_str()));
 
-  const vector<string>&commands=arg().getCommands();
+  const vector<string>&commands=::arg().getCommands();
 
   if(commands.empty()) {
     cerr<<"No command passed"<<endl;
@@ -89,8 +95,21 @@ int main(int argc, char **argv)
 
   try {
     string command=commands[0];
-
-    DynMessenger D(localdir,socketname);
+    shared_ptr<DynMessenger> D;
+    if(::arg()["remote-address"].empty())
+      D=shared_ptr<DynMessenger>(new DynMessenger(localdir,socketname));
+    else {
+      uint16_t port;
+      try {
+	port  = lexical_cast<uint16_t>(::arg()["remote-port"]);
+      }
+      catch(...) {
+	cerr<<"Unable to convert '"<<::arg()["remote-port"]<<"' to a port number for connecting to remote PowerDNS\n";
+	exit(99);
+      }
+      
+      D=shared_ptr<DynMessenger>(new DynMessenger(ComboAddress(::arg()["remote-address"], port), ::arg()["secret"]));
+    }
 
     string message;
     for(vector<string>::const_iterator i=commands.begin();i!=commands.end();++i) {
@@ -121,12 +140,12 @@ int main(int argc, char **argv)
     }
     
     
-    if(D.send(message)<0) {
+    if(D->send(message)<0) {
       cerr<<"Error sending command"<<endl;
       return 1;
     }
     
-    string resp=D.receive();
+    string resp=D->receive();
     
     cout<<resp<<endl;
   }
