@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2002 - 2005  PowerDNS.COM BV
+    Copyright (C) 2002 - 2007  PowerDNS.COM BV
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2 as 
@@ -23,7 +23,7 @@
 #include <iostream>
 #include <string>
 #include <sys/types.h>
-
+#include <boost/shared_ptr.hpp>
 #include "dns.hh"
 #include "dnsbackend.hh"
 #include "dnspacket.hh"
@@ -33,6 +33,8 @@
 #include "arguments.hh"
 #include "statbag.hh"
 
+using namespace boost;
+
 extern StatBag S;
 
 /** \mainpage 
@@ -40,7 +42,7 @@ extern StatBag S;
     own backend, see the documentation for the DNSBackend class.
 
     \section copyright Copyright and License
-    PowerDNS is (C) 2005 PowerDNS.COM BV. It is distributed according to the terms of the General Public License version 2.
+    PowerDNS is (C) 2001-2008 PowerDNS.COM BV. It is distributed according to the terms of the General Public License version 2.
 
     \section overview High level overview
 
@@ -77,7 +79,7 @@ extern StatBag S;
 void UDPNameserver::bindIPv4()
 {
   vector<string>locals;
-  stringtok(locals,arg()["local-address"]," ,");
+  stringtok(locals,::arg()["local-address"]," ,");
 
   if(locals.empty())
     throw AhuException("No local address specified");
@@ -91,6 +93,9 @@ void UDPNameserver::bindIPv4()
     if(s<0)
       throw AhuException("Unable to acquire a UDP socket: "+string(strerror(errno)));
   
+    if(locals.size() > 1 && !Utility::setNonBlocking(s))
+      throw AhuException("Unable to set UDP socket to non-blocking: "+stringerror());
+  
     memset(&locala,0,sizeof(locala));
     locala.sin_family=AF_INET;
 
@@ -109,15 +114,15 @@ void UDPNameserver::bindIPv4()
       locala.sin_addr.s_addr=*(int*)h->h_addr;
     }
 
-    locala.sin_port=htons(arg().asNum("local-port"));
+    locala.sin_port=htons(::arg().asNum("local-port"));
     
-    if(bind(s, (sockaddr*)&locala,sizeof(locala))<0) {
-      L<<Logger::Error<<"binding UDP socket to '"+localname+"': "<<strerror(errno)<<endl;
+    if(::bind(s, (sockaddr*)&locala,sizeof(locala))<0) {
+      L<<Logger::Error<<"binding UDP socket to '"+localname+"' port "+lexical_cast<string>(ntohs(locala.sin_port))+": "<<strerror(errno)<<endl;
       throw AhuException("Unable to bind to UDP socket");
     }
     d_highfd=max(s,d_highfd);
     d_sockets.push_back(s);
-    L<<Logger::Error<<"UDP server bound to "<<inet_ntoa(locala.sin_addr)<<":"<<arg().asNum("local-port")<<endl;
+    L<<Logger::Error<<"UDP server bound to "<<inet_ntoa(locala.sin_addr)<<":"<<::arg().asNum("local-port")<<endl;
     FD_SET(s, &d_rfds);
   }
 }
@@ -126,7 +131,7 @@ void UDPNameserver::bindIPv6()
 {
 #if !WIN32 && HAVE_IPV6
   vector<string>locals;
-  stringtok(locals,arg()["local-ipv6"]," ,");
+  stringtok(locals,::arg()["local-ipv6"]," ,");
 
   if(locals.empty())
     return;
@@ -143,15 +148,15 @@ void UDPNameserver::bindIPv6()
       L<<Logger::Warning<<"It is advised to bind to explicit addresses with the --local-ipv6 option"<<endl;
     }
     
-    ComboAddress locala(localname, arg().asNum("local-port"));
+    ComboAddress locala(localname, ::arg().asNum("local-port"));
 
-    if(bind(s, (sockaddr*)&locala, sizeof(locala))<0) {
+    if(::bind(s, (sockaddr*)&locala, sizeof(locala))<0) {
       L<<Logger::Error<<"binding to UDP ipv6 socket: "<<strerror(errno)<<endl;
       throw AhuException("Unable to bind to UDP ipv6 socket");
     }
     d_highfd=max(s,d_highfd);
     d_sockets.push_back(s);
-    L<<Logger::Error<<"UDPv6 server bound to ["<<localname<<"]:"<<arg().asNum("local-port")<<endl;
+    L<<Logger::Error<<"UDPv6 server bound to ["<<localname<<"]:"<<::arg().asNum("local-port")<<endl;
     FD_SET(s, &d_rfds);
   }
 #endif // WIN32
@@ -161,12 +166,12 @@ UDPNameserver::UDPNameserver()
 {
   d_highfd=0;
   FD_ZERO(&d_rfds);  
-  if(!arg()["local-address"].empty())
+  if(!::arg()["local-address"].empty())
     bindIPv4();
-  if(!arg()["local-ipv6"].empty())
+  if(!::arg()["local-ipv6"].empty())
     bindIPv6();
 
-  if(arg()["local-address"].empty() && arg()["local-ipv6"].empty()) 
+  if(::arg()["local-address"].empty() && ::arg()["local-ipv6"].empty()) 
     L<<Logger::Critical<<"PDNS is deaf and mute! Not listening on any interfaces"<<endl;
     
 }
@@ -175,13 +180,12 @@ void UDPNameserver::send(DNSPacket *p)
 {
   const char *buffer=p->getData();
   DLOG(L<<Logger::Notice<<"Sending a packet to "<< p->remote.toString() <<" ("<<p->len<<" octets)"<<endl);
-  if(p->len>512) {
-    p=new DNSPacket(*p);
-    p->truncate(512);
-    buffer=p->getData();
-    if(sendto(p->getSocket(),buffer,p->len,0,(struct sockaddr *)(&p->remote),p->remote.getSocklen())<0) 
-      L<<Logger::Error<<"Error sending reply with sendto (socket="<<p->getSocket()<<"): "<<strerror(errno)<<endl;
-    delete p;
+  if(p->len > p->getMaxReplyLen()) {
+    shared_ptr<DNSPacket> sharedp(new DNSPacket(*p));
+    sharedp->truncate(p->getMaxReplyLen());
+    buffer=sharedp->getData();
+    if(sendto(sharedp->getSocket(),buffer,sharedp->len,0,(struct sockaddr *)(&sharedp->remote), sharedp->remote.getSocklen())<0) 
+      L<<Logger::Error<<"Error sending reply with sendto (socket="<<sharedp->getSocket()<<"): "<<strerror(errno)<<endl;
   }
   else {
     if(sendto(p->getSocket(),buffer,p->len,0,(struct sockaddr *)(&p->remote),p->remote.getSocklen())<0)
