@@ -1,6 +1,6 @@
 /*
     PowerDNS Versatile Database Driven Nameserver
-    Copyright (C) 2005 - 2007  PowerDNS.COM BV
+    Copyright (C) 2005 - 2008  PowerDNS.COM BV
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2 
@@ -29,7 +29,9 @@
 #include <boost/algorithm/string.hpp>
 #include <boost/lexical_cast.hpp>
 
-ZoneParserTNG::ZoneParserTNG(const string& fname, const string& zname, const string& reldir) : d_reldir(reldir), d_zonename(zname), d_defaultttl(3600)
+ZoneParserTNG::ZoneParserTNG(const string& fname, const string& zname, const string& reldir) : d_reldir(reldir), 
+											       d_zonename(zname), d_defaultttl(3600), 
+											       d_havedollarttl(false)
 {
   d_zonename = toCanonic("", d_zonename);
   stackFile(fname);
@@ -40,14 +42,16 @@ void ZoneParserTNG::stackFile(const std::string& fname)
   FILE *fp=fopen(fname.c_str(), "r");
   if(!fp)
     throw runtime_error("Unable to open file '"+fname+"': "+stringerror());
-  d_fps.push(fp);
+
+  filestate fs(fp, fname);
+  d_filestates.push(fs);
 }
 
 ZoneParserTNG::~ZoneParserTNG()
 {
-  while(!d_fps.empty()) {
-    fclose(d_fps.top());
-    d_fps.pop();
+  while(!d_filestates.empty()) {
+    fclose(d_filestates.top().d_fp);
+    d_filestates.pop();
   }
 }
 
@@ -56,7 +60,23 @@ static string makeString(const string& line, const pair<string::size_type, strin
   return string(line.c_str() + range.first, range.second - range.first);
 }
 
-static unsigned int makeTTLFromZone(const string& str)
+static bool isTimeSpec(const string& nextpart)
+{
+  if(nextpart.empty())
+    return false;
+  for(string::const_iterator iter = nextpart.begin(); iter != nextpart.end(); ++iter) {
+    if(isdigit(*iter))
+      continue;
+    if(iter+1 != nextpart.end())
+      return false;
+    char c=tolower(*iter);
+    return (c=='m' || c=='h' || c=='d' || c=='w' || c=='y');
+  }
+  return true;
+}
+
+
+unsigned int ZoneParserTNG::makeTTLFromZone(const string& str)
 {
   if(str.empty())
     return 0;
@@ -80,8 +100,9 @@ static unsigned int makeTTLFromZone(const string& str)
     case 'Y': // ? :-)
       val*=3600*24*365;
       break;
+
     default:
-      throw ZoneParserTNG::exception("Unable to parse time specification '"+str+"'");
+      throw ZoneParserTNG::exception("Unable to parse time specification '"+str+"' "+getLineOfFile());
     }
   return val;
 }
@@ -197,6 +218,10 @@ bool findAndElide(string& line, char c)
   return false;
 }
 
+string ZoneParserTNG::getLineOfFile()
+{
+  return "on line "+lexical_cast<string>(d_filestates.top().d_lineno)+" of file '"+d_filestates.top().d_filename+"'";
+}
 
 bool ZoneParserTNG::get(DNSResourceRecord& rr) 
 {
@@ -204,7 +229,7 @@ bool ZoneParserTNG::get(DNSResourceRecord& rr)
   if(!getTemplateLine() && !getLine())
     return false;
 
-  chomp(d_line, " \r\n\x1a");
+  boost::trim_right_if(d_line, is_any_of(" \r\n\x1a"));
 
   parts_t parts;
   vstringtok(parts, d_line);
@@ -212,10 +237,15 @@ bool ZoneParserTNG::get(DNSResourceRecord& rr)
   if(parts.empty())
     goto retry;
 
+  if(parts[0].first != parts[0].second && makeString(d_line, parts[0])[0]==';') // line consisting of nothing but comments
+    goto retry;
+
   if(d_line[0]=='$') { 
     string command=makeString(d_line, parts[0]);
-    if(command=="$TTL" && parts.size() > 1)
-      d_defaultttl=makeTTLFromZone(makeString(d_line,parts[1]));
+    if(iequals(command,"$TTL") && parts.size() > 1) {
+      d_defaultttl=makeTTLFromZone(trim_right_copy_if(makeString(d_line, parts[1]), is_any_of(";")));
+      d_havedollarttl=true;
+    }
     else if(iequals(command,"$INCLUDE") && parts.size() > 1) {
       string fname=unquotify(makeString(d_line, parts[1]));
       if(!fname.empty() && fname[0]!='/' && !d_reldir.empty())
@@ -239,7 +269,7 @@ bool ZoneParserTNG::get(DNSResourceRecord& rr)
       goto retry;
     }
     else
-      throw exception("Can't parse zone line '"+d_line+"'");
+      throw exception("Can't parse zone line '"+d_line+"' "+getLineOfFile());
     goto retry;
   }
 
@@ -260,7 +290,7 @@ bool ZoneParserTNG::get(DNSResourceRecord& rr)
   d_prevqname=rr.qname;
 
   if(parts.empty()) 
-    throw exception("Line with too little parts");
+    throw exception("Line with too little parts "+getLineOfFile());
 
   string nextpart;
   
@@ -284,7 +314,7 @@ bool ZoneParserTNG::get(DNSResourceRecord& rr)
       // cout<<"Ignoring 'IN'\n";
       continue;
     }
-    if(!haveTTL && !haveQTYPE && all(nextpart, is_digit())) {
+    if(!haveTTL && !haveQTYPE && isTimeSpec(nextpart)) {
       rr.ttl=makeTTLFromZone(nextpart);
       haveTTL=true;
       // cout<<"ttl is probably: "<<rr.ttl<<endl;
@@ -300,15 +330,18 @@ bool ZoneParserTNG::get(DNSResourceRecord& rr)
       continue;
     }
     catch(...) {
-      throw runtime_error("Parsing zone content line: '"+nextpart+"' doesn't look like a qtype, stopping loop");
+      throw runtime_error("Parsing zone content "+getLineOfFile()+
+			  ": '"+nextpart+
+			  "' doesn't look like a qtype, stopping loop");
     }
   }
   if(!haveQTYPE) 
-    throw exception("Malformed line '"+d_line+"'");
+    throw exception("Malformed line "+getLineOfFile()+": '"+d_line+"'");
 
   rr.content=d_line.substr(range.first);
 
   chopComment(rr.content);
+  trim(rr.content);
 
   if(equals(rr.content, "@"))
     rr.content=d_zonename;
@@ -316,7 +349,7 @@ bool ZoneParserTNG::get(DNSResourceRecord& rr)
   if(findAndElide(rr.content, '(')) {      // have found a ( and elided it
     if(!findAndElide(rr.content, ')')) {
       while(getLine()) {
-	chomp(d_line,"\r\n ");
+	trim_right(d_line);
 	chopComment(d_line);
 	trim(d_line);
 	
@@ -349,10 +382,14 @@ bool ZoneParserTNG::get(DNSResourceRecord& rr)
     for(string::size_type n = 0; n < soaparts.size(); ++n) {
       if(n)
 	rr.content.append(1,' ');
+
       if(n > 1)
 	rr.content+=lexical_cast<string>(makeTTLFromZone(soaparts[n]));
       else
 	rr.content+=soaparts[n];
+
+      if(n==6 && !d_havedollarttl)
+	d_defaultttl=makeTTLFromZone(soaparts[n]);
     }
     break;
   default:;
@@ -364,14 +401,15 @@ bool ZoneParserTNG::get(DNSResourceRecord& rr)
 
 bool ZoneParserTNG::getLine()
 {
-  while(!d_fps.empty()) {
+  while(!d_filestates.empty()) {
     char buffer[1024];
-    if(fgets(buffer, 1024, d_fps.top())) {
+    if(fgets(buffer, 1024, d_filestates.top().d_fp)) {
+      d_filestates.top().d_lineno++;
       d_line=buffer;
       return true;
     }
-    fclose(d_fps.top());
-    d_fps.pop();
+    fclose(d_filestates.top().d_fp);
+    d_filestates.pop();
   }
   return false;
 }
